@@ -108,7 +108,23 @@ class PleaseRefresh(ExternalError):
     code: int = 1357004
 
 
-def handle_payload_error(j):
+@attr.s(slots=True, auto_exc=True, auto_attribs=True)
+class ServerRedirect(FacebookError):
+    """Raised by Facebook if the client is suspicious and the user needs to verify the login."""
+
+
+def handle_payload_error(j, ignore_jsmod_redirect: bool = False):
+    if not ignore_jsmod_redirect:
+        try:
+            jsmods_require_raw = j["jsmods"]["require"]
+            # Import here to avoid cyclic imports
+            from ._util import get_jsmods_require
+            jsmods_require = get_jsmods_require(jsmods_require_raw)
+            url = jsmods_require["ServerRedirect.redirectPageTo"][0]
+            raise ServerRedirect(f"Got server redirect to {url}, "
+                                 f"you may need to accept the session manually")
+        except (KeyError, IndexError):
+            pass
     if "error" not in j:
         return
     code = j["error"]
