@@ -1,16 +1,20 @@
 import attr
-import time
 import random
 import paho.mqtt.client
+import urllib.request
 import asyncio
 import aiohttp
 from ._common import log, kw_only
-from . import _util, _exception, _session, _graphql, _events
+from . import _util, _exception, _session, _events
 
 from typing import AsyncGenerator, Optional, List
 
+from yarl import URL
 
-HOST = "edge-chat.messenger.com"
+try:
+    import socks
+except ImportError:
+    socks = None
 
 TOPICS = [
     # Things that happen in chats (e.g. messages)
@@ -53,7 +57,7 @@ TOPICS = [
 def get_cookie_header(session: aiohttp.ClientSession, url: str) -> str:
     """Extract a cookie header from a requests session."""
     # The cookies are extracted this way to make sure they're escaped correctly
-    return session.cookie_jar.filter_cookies(url).output(header="", sep=";").lstrip()
+    return session.cookie_jar.filter_cookies(URL(url)).output(header="", sep=";").lstrip()
 
 
 def generate_session_id() -> int:
@@ -61,7 +65,7 @@ def generate_session_id() -> int:
     return random.randint(1, 2 ** 53)
 
 
-def mqtt_factory() -> paho.mqtt.client.Client:
+def mqtt_factory(domain: str) -> paho.mqtt.client.Client:
     # Configure internal MQTT handler
     mqtt = paho.mqtt.client.Client(
         client_id="mqttwsclient",
@@ -69,34 +73,31 @@ def mqtt_factory() -> paho.mqtt.client.Client:
         protocol=paho.mqtt.client.MQTTv31,
         transport="websockets",
     )
+    try:
+        http_proxy = urllib.request.getproxies()["http"]
+    except KeyError:
+        http_proxy = None
+    if http_proxy and socks and URL:
+        proxy_url = URL(http_proxy)
+        proxy_type = {
+            "http": socks.HTTP,
+            "https": socks.HTTP,
+            "socks": socks.SOCKS5,
+            "socks5": socks.SOCKS5,
+            "socks4": socks.SOCKS4,
+        }[proxy_url.scheme]
+        mqtt.proxy_set(proxy_type=proxy_type, proxy_addr=proxy_url.host, proxy_port=proxy_url.port,
+                       proxy_username=proxy_url.user, proxy_password=proxy_url.password)
     mqtt.enable_logger()
     # mqtt.max_inflight_messages_set(20)  # The rest will get queued
     # mqtt.max_queued_messages_set(0)  # Unlimited messages can be queued
     # mqtt.message_retry_set(20)  # Retry sending for at least 20 seconds
     # mqtt.reconnect_delay_set(min_delay=1, max_delay=120)
     mqtt.tls_set()
-    mqtt.connect_async(HOST, 443, keepalive=10)
+    mqtt.connect_async(f"edge-chat.{domain}", 443, keepalive=10)
     return mqtt
 
 
-async def fetch_sequence_id(session: _session.Session) -> int:
-    """Fetch sequence ID."""
-    params = {
-        "limit": 0,
-        "tags": ["INBOX"],
-        "before": None,
-        "includeDeliveryReceipts": False,
-        "includeSeqID": True,
-    }
-    log.debug("Fetching MQTT sequence ID")
-    # Same doc id as in `Client.fetch_threads`
-    (j,) = await session._graphql_requests(_graphql.from_doc_id("1349387578499440", params))
-    sequence_id = j["viewer"]["message_threads"]["sync_sequence_id"]
-    if not sequence_id:
-        raise _exception.NotLoggedIn("Failed fetching sequence id")
-    return int(sequence_id)
-
-
 @attr.s(slots=True, kw_only=kw_only, eq=False, auto_attribs=True)
 class Listener:
     """Listen to incoming Facebook events.
@@ -116,15 +117,16 @@ class Listener:
     _chat_on: bool
     _foreground: bool
     _loop: asyncio.AbstractEventLoop = attr.ib(factory=asyncio.get_event_loop)
-    _mqtt: paho.mqtt.client.Client = attr.ib(factory=mqtt_factory)
+    _mqtt: paho.mqtt.client.Client = None
     _disconnect_error: Optional[Exception] = None
     _sync_token: Optional[str] = None
     _sequence_id: Optional[int] = None
+    _sequence_id_wait: Optional[asyncio.Future] = None
     _tmp_events: List[_events.Event] = attr.ib(factory=list)
     _message_queue: asyncio.Queue = attr.ib(factory=lambda: asyncio.Queue(maxsize=64))
 
     def __attrs_post_init__(self):
-        # Configure callbacks
+        self._mqtt = mqtt_factory(self.session.domain)
         self._mqtt.on_message = self._on_message_handler
         self._mqtt.on_connect = self._on_connect_handler
         self._mqtt.on_socket_open = self.on_socket_open
@@ -290,11 +292,11 @@ class Listener:
 
         headers = {
             "Cookie": get_cookie_header(
-                self.session._session, "https://edge-chat.messenger.com/chat"
+                self.session._session, f"https://edge-chat.{self.session.domain}/chat"
             ),
             "User-Agent": self.session._session._default_headers["User-Agent"],
-            "Origin": "https://www.messenger.com",
-            "Host": HOST,
+            "Origin": f"https://www.{self.session.domain}",
+            "Host": f"edge-chat.{self.session.domain}",
         }
 
         # TODO: Is region (lla | atn | odn | others?) important?
@@ -315,6 +317,14 @@ class Listener:
         ) as e:
             raise _exception.NotLoggedIn("MQTT reconnection failed") from e
 
+    def set_sequence_id(self, sequence_id: int) -> None:
+        if self._sequence_id_wait:
+            log.debug("Got expected set_sequence_id call, waking up listener")
+            self._sequence_id_wait.set_result(sequence_id)
+            self._sequence_id_wait = None
+        else:
+            log.debug("Got unexpected set_sequence_id call")
+
     async def listen(self) -> AsyncGenerator[_events.Event, Optional[bool]]:
         """Run the listening loop continually.
 
@@ -328,8 +338,10 @@ class Listener:
             ...     print(event)
         """
         if self._sequence_id is None:
-            self._sequence_id = await fetch_sequence_id(self.session)
-        # TODO add some limits for all the infinite while loops
+            fut = self._sequence_id_wait = self._loop.create_future()
+            log.debug("Waiting for sequence ID...")
+            self._sequence_id = await fut
+            log.debug("Got sequence ID: %d", self._sequence_id)
 
         await self._reconnect()
         yield _events.Connect()
@@ -347,9 +359,12 @@ class Listener:
 
             # The sequence ID was reset in _handle_ms
             if self._sequence_id is None:
-                self._sequence_id = await fetch_sequence_id(self.session)
+                fut = self._sequence_id_wait = self._loop.create_future()
                 self._messenger_queue_publish()
                 yield _events.Resync()
+                log.debug("Waiting for sequence ID after resync...")
+                self._sequence_id = await fut
+                log.debug("Got sequence ID: %d", self._sequence_id)
 
             # If disconnect() has been called
             # Beware, internal API, may have to change this to something more stable!
