@@ -16,17 +16,21 @@ from http.cookies import SimpleCookie, BaseCookie
 # Or maybe just replace usage with `html.parser`?
 import bs4
 
-from ._common import log, kw_only
+from ._common import log, req_log, kw_only
 from . import _graphql, _util, _exception
 
 from typing import Optional, Mapping, Callable, Any, Awaitable, Dict, List, NamedTuple
 
 try:
-    from aiohttp_socks import ProxyType, ProxyConnector
+    from aiohttp_socks import ProxyType, ProxyConnector, ProxyTimeoutError
 except ImportError:
     ProxyType = None
     ProxyConnector = None
 
+
+    class ProxyTimeoutError(Exception):
+        pass
+
 SERVER_JS_DEFINE_REGEX = re.compile(r'(?:'
                                     r'\(new ServerJS\(\)\)(?:;s)?'
                                     r'|\(require\("ServerJS(?:Define)?"\)\)\(\)'
@@ -165,13 +169,17 @@ def get_user_id(domain: str, session: aiohttp.ClientSession) -> str:
 
 def session_factory(domain: str, user_agent: Optional[str] = None) -> aiohttp.ClientSession:
     from . import __version__
+    connector = None
     try:
         http_proxy = urllib.request.getproxies()["http"]
     except KeyError:
-        http_proxy = None
-    return aiohttp.ClientSession(connector=(ProxyConnector.from_url(http_proxy)
-                                            if ProxyConnector and http_proxy
-                                            else None),
+        pass
+    else:
+        if ProxyConnector:
+            connector = ProxyConnector.from_url(http_proxy)
+        else:
+            log.warning("http_proxy is set, but aiohttp-socks is not installed")
+    return aiohttp.ClientSession(connector=connector,
                                  headers={
                                      "Referer": f"https://www.{domain}/",
                                      "User-Agent": user_agent or f"fbchat-asyncio/{__version__}",
@@ -591,11 +599,19 @@ class Session:
             kwargs["headers"] = {"Host": real_url.host}
             kwargs["cookies"] = self._session.cookie_jar.filter_cookies(real_url)
             real_url = real_url.with_host(real_url.host.replace(self.domain, self._onion))
-        try:
-            r = await self._session.post(real_url, data=data, **kwargs)
-        except aiohttp.ClientError as e:
-            _exception.handle_requests_error(e)
-            raise Exception("handle_requests_error did not raise exception")
+        attempt = 1
+        while True:
+            try:
+                r = await self._session.post(real_url, data=data, **kwargs)
+                break
+            except aiohttp.ClientError as e:
+                _exception.handle_requests_error(e)
+                raise Exception("handle_requests_error did not raise exception")
+            except ProxyTimeoutError:
+                if attempt >= 3:
+                    raise
+                log.warning("Got ProxyTimeoutError, retrying...")
+                attempt += 1
         _exception.handle_http_error(r.status)
         text = await r.text()
         if text is None or len(text) == 0:
@@ -609,6 +625,10 @@ class Session:
             return j
 
     async def _payload_post(self, url, data, files=None):
+        if files:
+            req_log.debug("POST %s %s with %d files", url, data, len(files))
+        else:
+            req_log.debug("POST %s %s", url, data)
         j = await self._post(url, data, files=files)
         _exception.handle_payload_error(j)
 
@@ -632,6 +652,7 @@ class Session:
             "response_format": "json",
             "queries": _graphql.queries_to_json(*queries),
         }
+        req_log.debug("Making GraphQL queries: %s", queries)
         return await self._post("/api/graphqlbatch/", data, as_graphql=True)
 
     async def _do_send_request(self, data):
@@ -645,6 +666,8 @@ class Session:
         data["message_id"] = offline_threading_id
         data["threading_id"] = generate_message_id(now, self._client_id)
         data["ephemeral_ttl_mode:"] = "0"
+        req_log.debug("POST /messaging/send/ <data redacted>")
+        req_log.log(5, "Message data: %s", data)
         j = await self._post("/messaging/send/", data)
 
         _exception.handle_payload_error(j)
